local world

local ballA, ballB
local shapeA,shapeB

local aStuck, bStuck, aNormal, bNormal

local aFinished, bFinished

local worldBodies = {}
local killzones = {}
--local movers = {}

local debugText = ""

local titleFont, textFont, subFont

local worldScale = 10

local viewportX, viewportY = 0,0

local startX,startY = 0,0
local endX, endY = 0,0

local currentLevel, nextLevel

local levelBeganTime = 0
local levelIsSuccess, levelIsFailure = false, false

local showingHelp, showingInitiialHelp = false, false

-- images
local blackSpikes, blackNoSpikes, whiteSpikes, whiteNoSpikes, startPad, endPad, portalParticle, lightningPortLeft, lightningPortRight, wall, bg, ballHighlight, successText, failureText, controls, hTile, titleScreen

local startInnerSystem, startOuterSystem, endInnerSystem, endOuterSystem

local worldTime = 0

local bgMusic

local startedCreditTime = nil

local titleText, subtitleText

local gameTitleTime = nil
local initialHelpTime = nil

function load()
	titleFont = love.graphics.newFont("data/grenouille.ttf",80)
	textFont = love.graphics.newFont("data/yorkville.ttf",26)
	subFont = love.graphics.newFont("data/yorkville.ttf",20)

	world = love.physics.newWorld(worldScale*10,worldScale*100)
	world:setGravity(0,worldScale*3)
	
	ballA = love.physics.newBody(world,0,0)
	ballB = love.physics.newBody(world,0,0)
	
	ballA:setAngularDamping(10)
	ballB:setAngularDamping(10)
	
	shapeA = love.physics.newCircleShape(ballA,19/worldScale)
	shapeA:setRestitution(0.3)
	shapeA:setFriction(0.1)
	
	shapeB = love.physics.newCircleShape(ballB,19/worldScale)
	shapeB:setRestitution(0.3)
	shapeB:setFriction(0.1)
	
	joint = love.physics.newDistanceJoint(ballA,ballB,ballA:getX(),ballA:getY(),ballB:getX(),ballB:getY())
	joint:setLength(80/worldScale)
	joint:setFrequency(1.5)
	
	ballA:setMassFromShapes()
	ballB:setMassFromShapes()
	
	bgMusic = love.audio.newMusic("data/bgmusic.ogg")
	love.audio.play(bgMusic,0)
	
	gameTitleTime = love.timer.getTime()
	
	world:setCallback(collision)
	
	love.graphics.setBackgroundColor(220,220,220)
	
	blackSpikes = love.graphics.newImage("images/ball-black-spike.png")
	blackNoSpikes = love.graphics.newImage("images/ball-black-nospike.png")
	whiteSpikes = love.graphics.newImage("images/ball-white-spike.png")
	whiteNoSpikes = love.graphics.newImage("images/ball-white-nospike.png")
	startPad = love.graphics.newImage("images/start-pad.png")
	endPad = love.graphics.newImage("images/end-pad.png")
	lightningPortLeft = love.graphics.newImage("images/lightning-port-left.png")
	lightningPortRight = love.graphics.newImage("images/lightning-port-right.png")
	wall = love.graphics.newImage("images/wall.png")
	bg = love.graphics.newImage("images/bg.png")
	ballHighlight = love.graphics.newImage("images/ball-highlight.png")
	successText = love.graphics.newImage("images/success-text.png")
	failureText = love.graphics.newImage("images/failure-text.png")
	controls = love.graphics.newImage("images/controls.png")
	hTile = love.graphics.newImage("images/floor.png")
	titleScreen = love.graphics.newImage("images/title-screen.png")
	portalParticle = love.graphics.newImage("images/portal-particle.png")
	
	local sI = love.graphics.newParticleSystem(portalParticle,300)
	sI:setEmissionRate(30)
	sI:setLifetime(-1)
	sI:setParticleLife(0.6,1.4)
	sI:setSpeed(20)
	sI:setSize(0.8,0.4)
	sI:setSpread(360)
	sI:setColor(love.graphics.newColor(220,0,255,160),love.graphics.newColor(220,0,255,40))
	sI:start()
	
	local sO = love.graphics.newParticleSystem(portalParticle,600)
	sO:setEmissionRate(70)
	sO:setLifetime(-1)
	sO:setParticleLife(0.6,1.4)
	sO:setSpeed(70)
	sO:setRadialAcceleration(-80)
	sO:setSize(0.4,1.2)
	sO:setSpread(360)
	sO:setColor(love.graphics.newColor(0,240,255,255),love.graphics.newColor(0,240,255,80))
	sO:start()
	
	startInnerSystem = sI
	startOuterSystem = sO
	
	local eI = love.graphics.newParticleSystem(portalParticle,300)
	eI:setEmissionRate(30)
	eI:setLifetime(-1)
	eI:setParticleLife(0.6,1.4)
	eI:setSpeed(20)
	eI:setSize(0.8,0.4)
	eI:setSpread(360)
	eI:setColor(love.graphics.newColor(255,180,0,200),love.graphics.newColor(255,180,0,40))
	eI:start()
	
	local eO = love.graphics.newParticleSystem(portalParticle,600)
	eO:setEmissionRate(50)
	eO:setLifetime(-1)
	eO:setParticleLife(0.6,1.4)
	eO:setSpeed(70)
	eO:setRadialAcceleration(-80)
	eO:setSize(0.4,1.2)
	eO:setSpread(360)
	eO:setColor(love.graphics.newColor(140,0,255,255),love.graphics.newColor(140,0,255,80))
	eO:start()
	
	endInnerSystem = eI
	endOuterSystem = eO
end

function loadLevel(name)
	destroyLevel()
	currentLevel = name
	aStuck = false
	bStuck = false
	ballA:setVelocity(0,0)
	ballB:setVelocity(0,0)
	love.filesystem.include("levels/" .. name .. ".lua")
	levelBeganTime = love.timer.getTime()
	levelIsSuccess = false
	levelIsFailure = false
end

function destroyLevel()
	for _, b in pairs(worldBodies) do
		b.shape:destroy()
		b.body:destroy()
	end
	worldBodies = {}

	killzones = {}
	nextLevel = nil
end

function failed()
	loadLevel(currentLevel)
	levelIsFailure = true
end

function succeeded()
	if nextLevel then
		
		loadLevel(nextLevel)
		levelIsSuccess = true
	else
		startedCreditTime = love.timer.getTime()
	end
end

function setStageTitle(title,sub)
	titleText = title
	subtitleText = sub
end

function placeBalls(x1,y1,x2,y2)
	ballA:setPosition(x1*worldScale,y1*worldScale)
	ballB:setPosition(x2*worldScale,y2*worldScale)
	startX = (x1+x2)/2
	startY = (y1+y2)/2
end

function placeStartpoint(x,y)
	ballA:setPosition((x - .5)*worldScale, y*worldScale)
	ballB:setPosition((x + .5)*worldScale, y*worldScale)
	startX = x
	startY = y
end

function placeEndpoint(x,y)
	endX = x
	endY = y
end

function showInitialHelp()
	levelBeganTime = -4
	showingInitialHelp = true
	initialHelpTime = love.timer.getTime()
end

function hideInitialHelp()
	levelBeganTime = love.timer.getTime() - 1
	showingInitialHelp = false
	initialHelpTime = nil
end

-- update everything

function update(dt)
	if love.keyboard.isDown(love.key_space) then
		if gameTitleTime and gameTitleTime + 0.8 < love.timer.getTime() then
			gameTitleTime = nil
			loadLevel("first")
			showInitialHelp()
			worldTime = 0
		end
		if startedCreditTime and startedCreditTime + 3 < love.timer.getTime() then
			startedCreditTime = nil
			gameTitleTime = love.timer.getTime()
		end
	end
	if startedCreditTime or gameTitleTime then return end
	if love.keyboard.isDown(love.key_r) then loadLevel(currentLevel) end
	if love.keyboard.isDown(love.key_space) and (not initialHelpTime or love.timer.getTime() > initialHelpTime + 0.6)then hideInitialHelp() end
	showingHelp = showingInitialHelp or love.keyboard.isDown(love.key_h)
	if showingHelp then return end
	
	local oldXa, oldYa, oldXb, oldYb
	oldXa = ballA:getX()
	oldYa = ballA:getY()
	oldXb = ballB:getX()
	oldYb = ballB:getY()
	
	worldTime = worldTime + dt
	
	if worldTime > 300 then
		startedCreditTime = love.timer.getTime()
	end
	
	for _, k in pairs(killzones) do
		if k.phase then -- it turns on and off
			local t = math.floor(love.timer.getTime()*.7 + k.phase) % 2
			k.on = (t == 0)
		end
		local w,h
		if k.v then w = 0.6 h = k.sz else w = k.sz h = 0.6 end
		if ((math.abs(oldXa - k.x*worldScale) < w*worldScale/2 and math.abs(oldYa - k.y*worldScale) < h*worldScale/2) or (math.abs(oldXb - k.x*worldScale) < w*worldScale/2 and math.abs(oldYb - k.y*worldScale) < h*worldScale/2)) and ((not k.phase) or k.on) then
			failed()
			return
		end
	end
	
	if length(oldXa - endX*worldScale, oldYa - endY*worldScale) < 3 or length(oldXb - endX*worldScale, oldYb - endY*worldScale) < 3 then
		succeeded()
		return
	end
	
	local jumpForce = 50000
	
	if love.keyboard.isDown(love.key_w) and aStuck then
		aStuck = false
		local xAdd, yAdd = 0,0
		if love.keyboard.isDown(love.key_a) then
			xAdd = aNormal[2]
			yAdd = aNormal[1]
		elseif love.keyboard.isDown(love.key_d) then
			xAdd = -aNormal[2]
			yAdd = aNormal[1]
		end
		ballA:applyForce(aNormal[1]*jumpForce + xAdd*jumpForce*.3,aNormal[2] * jumpForce + yAdd*jumpForce*.3)
	end
	
	
	if love.keyboard.isDown(love.key_i) and bStuck then
		bStuck = false
		local xAdd, yAdd = 0,0
		if love.keyboard.isDown(love.key_j) then
			xAdd = bNormal[2]
			yAdd = bNormal[1]
		elseif love.keyboard.isDown(love.key_l) then
			xAdd = -bNormal[2]
			yAdd = bNormal[1]
		end
		
		ballB:applyForce(bNormal[1]*jumpForce + xAdd*jumpForce*.3,bNormal[2]*jumpForce + yAdd*jumpForce*.3)
	end
	if (aNormal and bNormal) and not ((aNormal[2] < 0 and math.abs(aNormal[1]) < aNormal[2]) and (bNormal[2] < 0 and math.abs(bNormal[1]) < bNormal[2])) then
	if love.keyboard.isDown(love.key_s) then
		aStuck = false
	end
	if love.keyboard.isDown(love.key_k) then
		bStuck = false
	end
	end
	
	world:update(math.min(dt,0.02))
	
	if aStuck then
		ballA:setPosition(oldXa,oldYa)
		ballA:setVelocity(0,0)
		ballA:setSpin(0)
	end
	if bStuck then
		ballB:setPosition(oldXb,oldYb)
		ballB:setVelocity(0,0)
		ballB:setSpin(0)
	end
	
	startInnerSystem:update(dt)
	startOuterSystem:update(dt)
	endInnerSystem:update(dt)
	endOuterSystem:update(dt)
end

function setNextLevel(name)
	nextLevel = name
end

-- draws everything

function draw()
	if not gameTitleTime then
		viewportX = viewportX * 0.9 + (-400 + ((ballA:getX() + ballB:getX()) / 2) * worldScale) * 0.1
		viewportY = viewportY * 0.9 + (-300 + ((ballA:getY() + ballB:getY()) / 2) * worldScale) * 0.1
		
		-- background
		for x = 0,800,40 do
			for y = 0,600,40 do
				love.graphics.draw(bg,x,y)
			end
		end
		
		-- portals
		
		love.graphics.draw(startPad,startX*worldScale*10 - viewportX,(startY + .5)*worldScale*10 - viewportY)
		love.graphics.draw(endPad,endX*worldScale*10 - viewportX,(endY+.5)*worldScale*10 - viewportY)
		
		love.graphics.setColorMode(love.color_modulate)
		love.graphics.draw(startOuterSystem,startX*worldScale*10 - viewportX,startY*worldScale*10 - viewportY)
		love.graphics.draw(startInnerSystem,startX*worldScale*10 - viewportX,startY*worldScale*10 - viewportY)
		love.graphics.draw(endOuterSystem,endX*worldScale*10 - viewportX,endY*worldScale*10 - viewportY)
		love.graphics.draw(endInnerSystem,endX*worldScale*10 - viewportX,endY*worldScale*10 - viewportY)
		love.graphics.setColorMode(love.color_normal)
		
		-- balls
		
		local aX = ballA:getX() * worldScale - viewportX
		local aY = ballA:getY() * worldScale - viewportY
		local bX = ballB:getX() * worldScale - viewportX
		local bY = ballB:getY() * worldScale - viewportY
		
		love.graphics.setColor(255,255,255)
		love.graphics.line(aX,aY,bX,bY)
		love.graphics.setColor(230,180,255)
		drawString(aX,aY,bX,bY,6)
		drawString(aX,aY,bX,bY,-6)
		
		love.graphics.setColor(220,80,255)
		love.graphics.setLineWidth(2)
		drawString(aX,aY,bX,bY,10)
		drawString(aX,aY,bX,bY,-10)
		love.graphics.setColor(200,0,255)
		drawString(aX,aY,bX,bY,15)
		drawString(aX,aY,bX,bY,-15)
		love.graphics.setLineWidth(2)
		
		local aImg, bImg
		if aStuck then aImg = blackSpikes else aImg = blackNoSpikes end
		if bStuck then bImg = whiteSpikes else bImg = whiteNoSpikes end
		
		love.graphics.draw(aImg,aX,aY,ballA:getAngle())
		love.graphics.draw(ballHighlight,aX,aY)
		love.graphics.draw(bImg,bX,bY,ballB:getAngle())
		love.graphics.draw(ballHighlight,bX,bY)
		
		-- killzones
		love.graphics.setColor(255,40 + 20*math.sin(love.timer.getTime() * 5),0)
		for _, k in pairs(killzones) do
			local x1,y1,x2,y2
			local w,h = 1,1
			if k.v then w = 0.2 h = k.sz else w = k.sz h = 0.2 end
			if k.v then
				x1 = k.x*worldScale*10-viewportX
				x2 = x1
				y1 = (k.y - k.sz/2) * worldScale*10 - viewportY
				y2 = y1 + k.sz * worldScale*10
			else
				y1 = k.y*worldScale*10-viewportY
				y2 = y1
				x1 = (k.x - k.sz/2) * worldScale*10 - viewportX
				x2 = x1 + k.sz*worldScale*10
			end
				
			if not (k.phase and not k.on) then
				drawLightning(x1,y1,x2,y2,2,love.graphics.newColor(255,150,0))
				drawLightning(x1,y1,x2,y2,6,love.graphics.newColor(255,230,0))
			end
			
			if k.v then
				love.graphics.draw(lightningPortLeft,x1,y1,90)
				love.graphics.draw(lightningPortRight,x1,y2,90)
			else
				love.graphics.draw(lightningPortLeft,x1 + 12,y1)
				love.graphics.draw(lightningPortRight,x2 - 12,y1)
			end
		end

		-- world geometry
		love.graphics.setColor(0,0,0)
		for _, b in pairs(worldBodies) do
			
			if b.w < b.h then
				love.graphics.setScissor((b.x)*worldScale*10 - viewportX - 40,(b.y - b.h/2)*worldScale*10 - viewportY,80,b.h*worldScale*10)
				local startY = (b.y - b.h/2) * worldScale * 10
				local endY = (b.y + b.h/2) * worldScale * 10
				local dist = (endY - startY)
				dist = (dist - (dist%100)) + 100
				endY = startY + dist
				for y = startY,endY,100 do
					love.graphics.draw(wall,b.x*worldScale*10 - viewportX,y - viewportY)
				end
			else
				love.graphics.setScissor((b.x - b.w/2)*worldScale*10 - viewportX,(b.y - b.h/2)*worldScale*10 - viewportY,b.w*worldScale*10,b.h*worldScale*10)
				local startX = (b.x - b.w/2) * worldScale * 10
				local endX = (b.x + b.w/2) * worldScale * 10
				local dist = (endX - startX)
				dist = (dist - (dist%60)) + 60
				endX = startX + dist
				for x = startX,endX,60 do
					love.graphics.draw(hTile,x - viewportX,b.y*worldScale*10 - viewportY - 15)
					love.graphics.draw(hTile,x - viewportX,b.y*worldScale*10 - viewportY + 15)
				end
			end
		end
		love.graphics.setScissor()
		
		local t = math.ceil(300 - worldTime)
		local tS = t % 60
		local tM = (t - tS) / 60
		love.graphics.setFont(textFont)
		local padZero = ""
		if tS < 10 then padZero="0" end
		t = tostring(tM) .. ":" .. padZero .. tostring(tS)
		love.graphics.setColor(0,0,0,180)
		love.graphics.drawf(t,40,562,100,love.align_left)
		love.graphics.setColor(255,255,255,200)
		love.graphics.drawf(t,40,560,100,love.align_left)
		
		local levelFadeFrac = (love.timer.getTime() - levelBeganTime) / 1.2
		if levelFadeFrac < 1 then
			love.graphics.setColor(0,0,0,255 * math.pow(1 - levelFadeFrac,4))
			love.graphics.rectangle(love.draw_fill,-5,-5,810,610)
		end
		
		if levelFadeFrac < 1.6 and (levelIsSuccess or levelIsFailure) then
			local titleFrac = levelFadeFrac / 1.6
			
			local a = (1 - math.pow(titleFrac,5))
			local vOff = math.pow(titleFrac,10)
			
			local img = successText
			if levelIsFailure then img = failureText end
			
			love.graphics.setColorMode(love.color_modulate)
				love.graphics.setColor(255,255,255,255*a)
				love.graphics.draw(img,400,250 - 60*vOff)
			love.graphics.setColorMode(love.color_normal)
		end
		
		if levelFadeFrac > 1.0 then
			local titleFadeFrac = (levelFadeFrac - 1) / 2.0
			if titleFadeFrac < 1 then
				local a = math.pow(math.sin(titleFadeFrac*math.pi*2),0.8)
				
				love.graphics.setFont(titleFont)
				love.graphics.setColor(0,0,0,180*a)
				love.graphics.drawf(titleText,200,103,400,love.align_center)
				love.graphics.setColor(255,255,255,255*a)
				love.graphics.drawf(titleText,200,100,400,love.align_center)
				
				love.graphics.setFont(textFont)
				love.graphics.drawf(subtitleText,100,140,600,love.align_center)
			end
		end
	end
	
	if startedCreditTime then
		local cAppFrac = (love.timer.getTime() - startedCreditTime) / 0.4
		if cAppFrac > 1 then cAppFrac = 1 end
		love.graphics.setColor(0,0,0,255*(1 - math.pow(1 - cAppFrac,4)))
		love.graphics.rectangle(love.draw_fill,-5,-5,810,610)
		
		local cTFrac = (love.timer.getTime() - startedCreditTime) / 1.0
		if cTFrac > 1 then cTFrac = 1 end
		local a = math.pow(cTFrac,6)
		love.graphics.setColor(255,255,255,255*a)
		love.graphics.setFont(titleFont)
		love.graphics.drawf("Thunder Balls",100,110,600,love.align_center)
		
		
		love.graphics.setColor(255,255,255,255*a)
		love.graphics.setFont(textFont)
		love.graphics.drawf("a Team Team-Team production\nfor Global Game Jam 2009",150,190,500,love.align_center)
		
		love.graphics.setColor(180,180,180,255*a)
		love.graphics.drawf("programming and design",90,300,300,love.align_right)
		love.graphics.drawf("art and level design",90,360,300,love.align_right)
		love.graphics.drawf("game design and interface art",40,420,350,love.align_right)
		love.graphics.setColor(255,255,255,255*a)
		love.graphics.drawf("Noah Witherspoon",410,300,300,love.align_left)
		love.graphics.drawf("Vu Ha",410,360,300,love.align_left)
		love.graphics.drawf("Redbeard Justice",410,420,300,love.align_left)
		
		love.graphics.setFont(subFont)
		love.graphics.setColor(180,180,180,255*a)
		love.graphics.drawf("Music: \"Ready to Go Up Higher\", by Macroform via OPSOUND",200,480,400,love.align_center)
		
		love.graphics.setColor(255,255,255,255*a)
		love.graphics.drawf("press Space to continue",200,560,400,love.align_center)
	end
	
	if showingHelp then
		love.graphics.setColor(0,0,0,150)
		love.graphics.rectangle(love.draw_fill,-5,-5,810,610)
		love.graphics.draw(controls,400,300)
		love.graphics.setColor(255,255,255,255)
		love.graphics.setFont(subFont)
		if showingInitialHelp then
			love.graphics.drawf("To show this page again later, hold the H key.\nTo continue, press Space.",100,500,600,love.align_center)
		end
	end
	
	if gameTitleTime then
		local tFrac = love.timer.getTime() - gameTitleTime
		love.graphics.draw(titleScreen,400,300)
		if tFrac < 1 then
			love.graphics.setColor(0,0,0,255*math.pow(1 - tFrac,4))
			love.graphics.rectangle(love.draw_fill,-5,-5,810,610)
		end
		love.graphics.setColor(255,255,255,255)
		love.graphics.setFont(subFont)
		love.graphics.drawf("press Space to begin",200,560,400,love.align_center)
	end
end


-- physics collision callback (obj1 and obj2 are always empty, for reasons of bugs and suck)

function collision(obj1,obj2,collData)
	local cX,cY = collData:getPosition()
	
	local distA = bodyDist(ballA,cX,cY)
	local distB = bodyDist(ballB,cX,cY)
	
	local nX,nY = collData:getNormal()
	
	if distA <= 2.5 then
		aStuck = true
		aNormal = {nX,nY}
	elseif distB <= 2.5 then
		bStuck = true
		bNormal = {nX,nY}
	end
end


-- draw a parabola "string" between two objects

function drawString(x1,y1,x2,y2, mul)
	local offX = x2 - x1
	local offY = y2 - y1
	local iM = -offX / offY
	local lastLineX, lastLineY = x1,y1
	local offLen = length(offX, offY)
	
	local co1,co2,co3,co4 = math.random()*10,math.random(),math.random()*10,math.random()
	
	local zag = function(t) return math.sin(t*co1 + co2) + math.sin(t*co3 + co4)*0.1 end
	
	for i=0,1,0.1 do
		local thickness = (math.pow(i * 2 - 1,2)*.6 + .1*zag(i)) * mul
		local lineX = x1 + offX * i + (offY / offLen) * thickness
		local lineY = y1 + offY * i - (offX / offLen) * thickness
		love.graphics.line(lastLineX, lastLineY, lineX, lineY)
		lastLineX = lineX
		lastLineY = lineY
	end
end

function drawLightning(x1,y1,x2,y2,mul,color)
	if x1 == nil or y1 == nil or x2 == nil or y2 == nil or mul == nil then debugText = "Something's nil" return end
	local offX = x2 - x1
	local offY = y2 - y1
	local iM = -offX / offY
	local lastLineX, lastLineY = x1,y1
	local offLen = length(offX, offY)
	
	local co1, co2, co3 = 14 + math.random()*6, 12 + math.random()*8, math.random()*4
	local noise = function(t) return (math.asin(math.sin(t*co1 - co3)) + math.cos(t*co2)*0.5) end
	
	local lastLineX, lastLineY
	
	for i=0,1,0.01 do
		--local x,y = x1 + i*(k.sz*worldScale*10), y1+noise(i)*5
		
		local zag = noise(i) * mul
		
		local lineX = x1 + offX * i + (offY / offLen) * zag
		local lineY = y1 + offY * i - (offX / offLen) * zag
		if lastLineX ~= nil and lastLineY ~= nil then
			color:setAlpha(160)
			love.graphics.setColor(color)
			love.graphics.setLineWidth(8)
			love.graphics.line(lastLineX,lastLineY,lineX,lineY)
			color:setAlpha(255)
			love.graphics.setColor(color)
			love.graphics.setLineWidth(4)
			love.graphics.line(lastLineX,lastLineY,lineX,lineY)
			love.graphics.setColor(255,255,255)
			love.graphics.setLineWidth(2)
			love.graphics.line(lastLineX,lastLineY,lineX,lineY)
		end
		lastLineX = lineX
		lastLineY = lineY
	end
end


-- adds a box to the world geometry

function addWorldBox(x,y,w,h)
	t = {}
	bod = love.physics.newBody(world, 0,0, 0)
	shp = love.physics.newRectangleShape(bod,x*100/worldScale,y*100/worldScale,w*100/worldScale,h*100/worldScale)
	shp:setRestitution(0.2)
	shp:setFriction(0.1)
	
	bod.w = w
	bod.h = h

	t.body = bod
	t.shape = shp
	t.x = x
	t.y = y
	t.w = w
	t.h = h
	
	worldBodies[#worldBodies + 1] = t
	return bod
end

function addKillzone(x,y,sz,v)
	local k = {}
	k.x = x
	k.y = y
	k.sz = sz
	k.v = v
	killzones[#killzones + 1] = k
	
	return k
end

function addOnoffKillzone(x,y,sz,v,p)
	local k = addKillzone(x,y,sz,v)
	k.phase = p
	return k
end

-- distance from a body to a position

function bodyDist(b,x,y)
	return length(b:getX() - x,b:getY() - y)
end

function length(x,y)
	return math.sqrt(x*x + y*y)
end
